"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const rd_1 = __importDefault(require("rd"));
const handlebars_1 = __importDefault(require("handlebars"));
const lodash_1 = __importDefault(require("lodash"));
const prettier_1 = __importDefault(require("prettier"));
const core_1 = __importDefault(require("@svgr/core"));
const glob_promise_1 = __importDefault(require("glob-promise"));
const string_hash_1 = __importDefault(require("string-hash"));
const readConfig_1 = __importDefault(require("../helper/readConfig"));
const chalk_1 = __importDefault(require("chalk"));
const env_1 = require("../config/env");
const { relative } = path_1.default;
const context = __dirname;
const config = (0, readConfig_1.default)();
if (env_1.nodeEnv === "dev") {
    console.log(config);
}
const commandPath = process.cwd();
function generateIcons() {
    // 获得当前执行node命令时候的文件夹目录名
    let template = fs_1.default.readFileSync(path_1.default.join(__dirname, "./template/taro.handlebars"), "utf-8");
    let templates = [];
    if (config.templateFile) {
        // 单模版模式
        const relativePath = path_1.default.join(commandPath, config.templateFile);
        try {
            template = fs_1.default.readFileSync(relativePath, "utf-8");
        }
        catch (error) {
            console.log(chalk_1.default.red(`🔥 图标模板文件不存在 `), relativePath);
        }
    }
    if (config.templates) {
        // 多模板模式
        config.templates.forEach((item) => {
            const relativePath = path_1.default.join(commandPath, item.templateFile);
            try {
                const templateContent = fs_1.default.readFileSync(relativePath, "utf-8");
                if (templateContent) {
                    templates.push({
                        fileType: item.fileType,
                        template: templateContent,
                    });
                }
            }
            catch (error) {
                console.log(chalk_1.default.red(`🔥 图标模板文件不存在 `), relativePath);
            }
        });
    }
    const routesPath = path_1.default.resolve(commandPath, config.iconRoot);
    // 同步遍历目录下的所有文件
    rd_1.default.eachSync(routesPath, async (fileDir, stats) => {
        const extname = path_1.default.extname(fileDir);
        const isValidPicture = stats.isFile() &&
            /svg|png|jpeg/.test(path_1.default.extname(fileDir)) &&
            !/@[23]x/.test(fileDir);
        // console.log(fileDir);
        if (isValidPicture) {
            const name = path_1.default.basename(fileDir, path_1.default.extname(fileDir));
            const iconFileName = path_1.default.basename(fileDir);
            // 转换为双驼峰写法
            const iconName = lodash_1.default.upperFirst(lodash_1.default.camelCase(name));
            const iconComponentName = config.prefix
                ? `${config.prefix}${iconName}`
                : iconName;
            let finalTemplate = template;
            if (templates.length > 0) {
                templates.some((item) => {
                    const reg = new RegExp(item.fileType);
                    if (reg.test(extname)) {
                        finalTemplate = item.template;
                        return true;
                    }
                    return false;
                });
            }
            let skipGenerator = false;
            if (config.svgr && /svg/.test(extname)) {
                skipGenerator = await svgrGenerator(fileDir, iconComponentName);
            }
            if (!skipGenerator) {
                generator(finalTemplate, iconComponentName, {
                    iconName,
                    iconFileName,
                    iconComponentName,
                });
            }
        }
    });
}
exports.default = generateIcons;
const prettierFormat = (content) => {
    const options = {
        printWidth: 120,
        tabWidth: 2,
        trailingComma: "all",
        semi: false,
        parser: "typescript",
    };
    return prettier_1.default.format(content, options);
};
const generator = (template, fileName, compileData) => {
    const code = handlebars_1.default.compile(template)(compileData);
    const formatCode = prettierFormat(code);
    const iconPath = path_1.default.join(config.iconTarget, `/${fileName}.tsx`);
    fs_1.default.writeFileSync(iconPath, formatCode, "utf-8");
};
const svgrGenerator = async (fileDir, componentName) => {
    // TODO: 判断是否被过滤
    if (config.svgr.filter) {
        try {
            const files = await (0, glob_promise_1.default)(config.svgr.filter);
            const skip = files.some((file) => {
                const absoluteFilePath = path_1.default.resolve(commandPath, file);
                return absoluteFilePath === fileDir;
            });
            if (skip) {
                return !skip;
            }
        }
        catch (error) {
            console.log(error);
        }
    }
    // TODO: 读取文件数据
    const svgCode = fs_1.default.readFileSync(fileDir, "utf-8");
    config.svgr.platform.forEach((platform) => {
        (0, core_1.default)(svgCode, {
            native: platform === "rn",
            dimensions: false,
            replaceAttrValues: {
                "#000": "{props.color}",
                "#000000": "{props.color}",
                currentColor: "{props.color}",
            },
            svgo: true,
            svgoConfig: {
                plugins: [
                    // { removeTitle: false },
                    // { removeViewBox: false },
                    {
                        cleanupIDs: {
                            prefix: `svg${(0, string_hash_1.default)(relative(context, svgCode))}`,
                        },
                    },
                ],
            },
            plugins: [
                "@svgr/plugin-svgo",
                "@svgr/plugin-jsx",
                "@svgr/plugin-prettier",
            ],
        }, { componentName: componentName }).then((code) => {
            const iconBaseName = platform === "h5"
                ? `${componentName}.tsx`
                : `${componentName}.${platform}.tsx`;
            const iconPath = path_1.default.join(config.iconTarget, `/${iconBaseName}`);
            fs_1.default.writeFileSync(iconPath, prettierFormat(code), "utf-8");
        });
    });
    return true;
};
